package com.minelittlepony.unicopia.block.cloud;

import org.jetbrains.annotations.Nullable;

import com.minelittlepony.unicopia.EquineContext;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;

import net.minecraft.block.BlockState;
import net.minecraft.block.ShapeContext;
import net.minecraft.block.StairsBlock;
import net.minecraft.entity.Entity;
import net.minecraft.entity.ai.pathing.NavigationType;
import net.minecraft.item.ItemPlacementContext;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.util.shape.VoxelShapes;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;

public class CloudStairsBlock extends StairsBlock implements CloudLike {
    private static final MapCodec<CloudStairsBlock> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
            BlockState.CODEC.fieldOf("base_state").forGetter(block -> block.baseBlockState),
            StairsBlock.createSettingsCodec()
    ).apply(instance, CloudStairsBlock::new));

    private final CloudBlock baseBlock;

    public CloudStairsBlock(BlockState baseState, Settings settings) {
        super(baseState, CloudLike.applyCloudProperties(settings));
        this.baseBlock = (CloudBlock)baseState.getBlock();
    }

    @Override
    public MapCodec<? extends StairsBlock> getCodec() {
        return CODEC;
    }

    @Override
    protected float getAmbientOcclusionLightLevel(BlockState state, BlockView world, BlockPos pos) {
        return baseBlock.getAmbientOcclusionLightLevel(state, world, pos);
    }

    @Override
    protected boolean isTransparent(BlockState state, BlockView world, BlockPos pos) {
        return baseBlock.isTransparent(state, world, pos);
    }

    @Override
    public void onEntityLand(BlockView world, Entity entity) {
        baseBlock.onEntityLand(world, entity);
    }

    @Override
    public void onLandedUpon(World world, BlockState state, BlockPos pos, Entity entity, float fallDistance) {
        baseBlock.onLandedUpon(world, state, pos, entity, fallDistance);
    }

    @Override
    protected void onEntityCollision(BlockState state, World world, BlockPos pos, Entity entity) {
        baseBlock.onEntityCollision(state, world, pos, entity);
    }

    @Override
    protected VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
        if (!baseBlock.canInteract(state, world, pos, EquineContext.of(context))) {
            return VoxelShapes.empty();
        }
        return super.getOutlineShape(state, world, pos, context);
    }

    @Override
    protected final VoxelShape getCullingShape(BlockState state, BlockView world, BlockPos pos) {
        return super.getOutlineShape(state, world, pos, ShapeContext.absent());
    }

    @Override
    protected VoxelShape getCollisionShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
        return this.collidable ? state.getOutlineShape(world, pos, context) : VoxelShapes.empty();
    }

    @Override
    @Nullable
    public BlockState getPlacementState(ItemPlacementContext context) {
        EquineContext equineContext = EquineContext.of(context);
        if (!baseBlock.canInteract(getDefaultState(), context.getWorld(), context.getBlockPos(), equineContext)) {
            return null;
        }
        return super.getPlacementState(context);
    }

    @Override
    protected boolean canReplace(BlockState state, ItemPlacementContext context) {
        return baseBlock.canReplace(state, context);
    }

    @Override
    protected boolean isSideInvisible(BlockState state, BlockState stateFrom, Direction direction) {
        return baseBlock.isSideInvisible(state, stateFrom, direction);
    }

    @Override
    protected boolean canPathfindThrough(BlockState state, NavigationType type) {
        return baseBlock.canPathfindThrough(state, type);
    }
}
